#!/usr/bin/env python
# -*- coding: utf-8 -*-

from gimpfu import *
import struct

def pntload(filename, raw_filename):
    #:param str filename: ファイルパス
    #:param str raw_filename: 選択ファイル名
    #:rtype: gimp.Image
    #:return: 読み込んだ画像
    fp = open(filename, 'rb')
    header = fp.read(4)
    if not header[0:4] == "SXTF":
        gimp.message("PNTファイルではありません")
        fp.close()
        return 0

    #ヘッダ・ブロックを読む
    header_length = struct.unpack('>I', fp.read(4))[0]
    HeaderBlocks = {"dumyyyyy":(0,0)}
    while (header_length > 0):
        dbname = fp.read(4)
        offset = struct.unpack('>I', fp.read(4))[0]
        dataLength = struct.unpack('>I', fp.read(4))[0]
        HeaderBlocks[dbname] = (offset, dataLength)
        header_length -= 12

    #画像サイズ
    size = HeaderBlocks['SIZE'][0]
    width = size>>16
    height = size & 0xffff

    #パレット
    palx = [0]*16
    offset = HeaderBlocks['PALT'][0]
    #dataLength = HeaderBlocks['PALT'][1] #32のはず
    fp.seek(offset)
    for i in range(16):
        palx[i] = struct.unpack('>H', fp.read(2))[0]
        i +=1
    
    #パレット計算用テーブル 5bit+1bit を 8bit に
    octa = [0]*64
    for i in range(64):
        octa[i] = int(round(i * 255 * 1.0 / 63))

    #GIMP用のパレットを計算するよ 16ビットカラーを24ビットに
    palg = [0]*16*3
    for i in range(16):
        g = ( (palx[i] & 0b1111100000000000)>>10 ) + (palx[i] & 0b0000000000000001)
        r = ( (palx[i] & 0b0000011111000000)>>5 ) + (palx[i] & 0b0000000000000001)
        b = palx[i] & 0b0000000000111111
        palg[i*3] = octa[r]
        palg[i*3+1] = octa[g]
        palg[i*3+2] = octa[b]

    #GIMPに画像を作成
    image = gimp.Image(width, height, INDEXED)
    image.filename = filename

    #パレット(カラーマップ)を設定
    pdb.gimp_image_set_colormap(image, 16*3, palg)

    #レイヤー作成
    layer = pdb.gimp_layer_new(image, width, height, INDEXEDA_IMAGE, filename, 100, NORMAL_MODE )

    #画像展開
    if ( width%2 ):
        w2 = width+1
    else:
        w2 = width
    plain = [0]*w2*height

    offset = HeaderBlocks['GR2Z'][0]
    dataLength = HeaderBlocks['GR2Z'][1]
    fp.seek(offset)
    buf = fp.read(dataLength)
    dp = 0
    for y in range( height ):
        x = 0
        while ( x < w2 ):
            shiji = ord(buf[dp])
            dp += 1
            if ( shiji < 0x80 ):         #shiji+1バイト分そのまま出力
                for i in range(shiji+1):
                    ps = ord( buf[dp] )
                    dp += 1
                    p1 = ps>>4
                    p2 = ps & 0b00001111
                    plain[y*w2+x]   = p1
                    plain[y*w2+x+1] = p2
                    i +=1
                    x +=2
            else:                        #次の1バイトを繰り返し
                ps = ord( buf[dp])
                dp += 1
                p1 = ps>>4
                p2 = ps & 0b00001111
                for i in range(257 - shiji):
                    plain[y*w2+x]   = p1
                    plain[y*w2+x+1] = p2
                    i +=1
                    x += 2
        y +=1
        gimp.progress_update(float(y)/float(height*2)) # プログレスバーの更新

    for y in range(height):
        for x in range(width):
            pixel = plain[y*w2+x]
            layer.set_pixel( x, y, (pixel,255) )

        #pdb.gimp_drawable_update( layer, 0, 0, width, height)
	gimp.progress_update(float(y+height)/float(height*2)) # プログレスバーの更新
    
    image.add_layer(layer, 0)

    return image

# 登録ハンドラ
def register_load_handlers():
    gimp.register_load_handler('python_fu_PNT_load', 'pnt', '')
    pdb['gimp-register-file-handler-mime']('python_fu_PNT_load', 'image/pnt')
#    pdb['gimp-register-thumbnail-loader']('python_fu_PNT_load', '')


register(
    "python_fu_PNT_load", #コマンド名
    "X68用PNT形式画像を読み込みます", #プロシージャブラウザに表示されるプラグインに関する情報
    "X68用PNT形式画像を読み込みます", #プラグインのヘルプ
    "Akira M", #プラグインの作成者
    "Akira M", #プラグインの著作権保有者
    "2017", #年
    "X68 PNT", #メニューの中でプラグインに使用されるラベル
    None, #プラグインで処理する対象となる画像のタイプ
    [   #input args. Format (type, name, description, default [, extra])
        (PF_STRING, 'filename', 'The name of the file to load', None),
        (PF_STRING, 'raw-filename', 'The name entered', None),
    ], #プラグインのメソッドのパラメーター
    [(PF_IMAGE, 'image', 'Output image')], #プラグインのメソッドの結果 Format (type, name, description) 
    pntload,
    on_query = register_load_handlers,
    menu = "<Load>",
)

main()
